@@ -4,6 +4,7 @@
 */
 
 #include "libcig.h"
+#include <errno.h>
 #include <string.h>
 #include <stddef.h>
 #include <stdio.h>
@@ -42,7 +43,6 @@ char *scheme2c_strcpy(scheme_value sstr)
 {
     char *result;
     int slen;
-    extern int errno;
 
     slen = STRING_LENGTH(sstr);
     result = Malloc(char, slen+1);
