@@ -36,7 +36,7 @@
 #include <dlfcn.h>
 #endif
 
-#ifdef HAVE_LIBGEN_H
+#if defined(HAVE_LIBGEN) && defined(HAVE_LIBGEN_H)
 #include <libgen.h>
 /* if we have pathfind, get the file name with $LD_LIBRARY_PATH or $S48_EXTERN_PATH */
 static char *shared_object_name(char *name)
@@ -130,6 +130,20 @@ lookup_dlsym(char *name, long *location)
   name++;
 #endif
 
+  /* find the name in the self process image - ie original scheme48
+     executable file */
+  if (!selfhandle)
+    selfhandle=dlopen((char*)0, S48_DLOPEN_MODE);
+  if (adr=dlsym(selfhandle, name)) {
+    *location = (long) adr;
+#ifdef DLDEBUG
+  printf(" %s:%d %s name='%s' in self at adr=%#x\n",
+	 __FILE__, __LINE__, __FUNCTION__,
+	 name, (long) adr);
+#endif /*DLDEBUG*/
+    return 1;
+  };
+
 /* perhaps i should scan the dlopened_handle from last to first,
    to find newest incarnation of symbol? in practice it should be faster
    to go from first (oldest) to last,
@@ -145,19 +159,6 @@ lookup_dlsym(char *name, long *location)
 #endif /*DLDEBUG*/
       return 1;
     };
-  /* find the name in the self process image - ie original scheme48
-     executable file */
-  if (!selfhandle)
-    selfhandle=dlopen((char*)0, S48_DLOPEN_MODE);
-  if (adr=dlsym(selfhandle, name)) {
-    *location = (long) adr;
-#ifdef DLDEBUG
-  printf(" %s:%d %s name='%s' in self at adr=%#x\n",
-	 __FILE__, __LINE__, __FUNCTION__,
-	 name, (long) adr);
-#endif /*DLDEBUG*/
-    return 1;
-  };
   /* can't find name so return 0 */
 #ifdef DLDEBUG
   printf(" %s:%d %s name='%s' not found\n",
@@ -362,7 +363,7 @@ really_dynamic_load( char *reloc_info_file, char *output_file, char *user_args )
   free(command);
 
   /* open the output file */
-  output_file_desc = fopen(output_file, "r");
+  output_file_desc = fopen(output_file, "rb");
   if (output_file_desc == NULL ) {
     fprintf(stderr, "unable to open output file\n");
     abort_load(NULL, NULL, output_file);
