@@ -569,7 +569,7 @@
     (newline port)
     (write-line "Copyright (c) 1993, 1994 by Richard Kelsey and Jonathan Rees."
 		port)
-    (write-line "Please report bugs to scheme-48-bugs@martigny.ai.mit.edu."
+    (write-line "Please report bugs to scheme-48-bugs@zurich.ai.mit.edu."
                 port)
     (if (not (batch-mode?))
 	(write-line "Type ,? (comma question-mark) for help." port))))
