@@ -105,7 +105,7 @@ extended_vm (long key, scheme_value value)
       FILE *new_stream;
 
       if (pstream == NULL) return UNDEFINED;
-      new_stream = fdopen(fd, key == 24 ? "r" : "w");
+      new_stream = fdopen(fd, key == 24 ? "rb" : "wb");
       if (new_stream == NULL) return UNDEFINED;
       fclose(*pstream);
       *pstream = new_stream;
