@@ -5,6 +5,7 @@
 #include <sys/types.h>
 #include <stdio.h>
 #include <dirent.h>
+#include <errno.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -14,8 +15,6 @@
 /* Make sure our exports match up w/the implementation: */
 #include "dirstuff1.h"
 
-extern int errno;
-
 /* Linked list of malloc'd entries. */
 struct scm_dirent_struct
     { char *fname;			/* File name */ 
