@@ -21,6 +21,7 @@
 */
 #include "sysdep.h"
 #include <stdio.h>
+#include <errno.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -33,8 +34,6 @@
 /* Make sure our exports match up w/the implementation: */
 #include "fdports1.h"
 
-extern int errno;
-
 /* Maps fd's to FILE*'s. */
 static FILE *fstar_cache[NUM_FDPORTS] = {NULL};
 
@@ -437,7 +436,7 @@ scheme_value read_delim(const char *delims, char *buf, int gobble,
     ** stdio FILE*. Yech.
     */
     if( IsChar(peekc) ) {
-	char c = EXTRACT_CHAR(peekc);
+	unsigned char c = EXTRACT_CHAR(peekc);
 	if( delims[c]  ) {		/* Is c in cset? */
 	    if( gobble ) *PortData_Peek(data) = SCHFALSE;
 	    *nread = 0;
