@@ -1,13 +1,33 @@
-;;; Copyright (c) 1993, 1994 by Olin Shivers.
+;;; Copyright (c) 1993, 1994 by Olin Shivers. See file COPYING.
 
 ;;; chase? true (the default) means if the file is a symlink, chase the link
 ;;; and report on the file it references. chase? = #f means check the actual
 ;;; file itself, even if it's a symlink.
-;;; writeable means (1) file exists & is writeable OR (2) file doesn't exist
-;;;     but directory is writeable.
 
+;;; (file-not-accessible? perms fd/port/fname)
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;; PERMS is 3 bits, not 9.
+;;; writeable means (1) file exists & is writeable OR (2) file doesn't exist
+;;;     and directory is writeable. That is, writeable means writeable or
+;;;     creatable.
+;;;
+;;; There's a Posix call, access(), that checks using the *real* uid, not
+;;; the effective uid, so that setuid programs can figure out if the luser
+;;; has perms. file-not-accessible? is defined in terms of the effective uid, 
+;;; so we can't use access().
+;;; 
+;;; This is a kind of bogus function. The only way to do a real check is to
+;;; try an open() and see if it flies. Otherwise, there's an obvious atomicity
+;;; problem. Also, we special case root, saying root always has all perms. But
+;;; not even root can write on a read-only filesystem, such as a CD ROM. In
+;;; this case, we'd blithely say the file was writeable -- there's no way to
+;;; check for a ROFS without doing an open(). We need a euid analog to
+;;; access(). Ah, well.
+;;; 
+;;; I also should define a family of real uid perm-checking calls.
+;;;
 ;;; Return values:
-;;; #f			Accessible
+;;; #f			Accessible in at least one of the requested ways.
 ;;; search-denied	Can't stat
 ;;; permission		File exists but is protected
 ;;; 			(also for errno/rofs)
@@ -16,27 +36,43 @@
 ;;;
 ;;; Otherwise, signals an error.
 
-(define (file-not-accessible? perms fd/port/fname . maybe-chase?)
+(define (file-not-accessible? perms fd/port/fname)
   (let ((uid (user-effective-uid)))
-    (and (not (zero? uid)) ; Root can do what he likes.
-	 (with-errno-handler ((err data)
-			      ((errno/acces) 'search-denied)
-			      ((errno/noent) 'nonexistent)
-			      ((errno/notdir) 'not-directory))
+    (with-errno-handler ((err data)
+			 ((errno/acces) 'search-denied)
+			 ((errno/notdir) 'no-directory)
 
-	   (and (let* ((info (apply file-info fd/port/fname maybe-chase?))
-		       (acc (file-info:mode info)))
-		  (cond ((= (file-info:uid info) (user-effective-uid)) ; User
-			 (zero? (bitwise-and acc (arithmetic-shift perms 6))))
+			 ;; If the file doesn't exist, we usually return
+			 ;; 'nonexistent, but we special-case writability
+			 ;; for the directory check.
+			 ((errno/noent)
+			  (and (or (zero? (bitwise-and perms 2))
+				   ;; This string? test *has* to return #t.
+				   ;; If fd/port/fname is an fd or a port,
+				   ;; we wouldn't get an errno/noent error!
+				   ;; Just being paranoid...
+				   (not (string? fd/port/fname))
+				   ;; OK, check to see if we can create
+				   ;; files in the directory.
+				   (file-not-accessible? 2
+				     (directory-as-file-name
+				       (file-name-directory fd/port/fname))))
+			       'nonexistent)))
+
+      (and (let* ((info (file-info fd/port/fname))
+		  (acc (file-info:mode info)))
+	     (cond ((zero? uid) #f)	; Root can do as he wishes.
+
+		   ((= (file-info:uid info) (user-effective-uid)) ; User
+		    (zero? (bitwise-and acc (arithmetic-shift perms 6))))
 		      
-			((= (file-info:gid info) (user-effective-gid)) ; Group
-			 (zero? (bitwise-and acc (arithmetic-shift perms 3))))
-			((memv (file-info:gid info) (user-supplementary-gids))
-			 (zero? (bitwise-and acc (arithmetic-shift perms 3))))
+		   ((or (= (file-info:gid info) (user-effective-gid)) ; Group
+			(memv (file-info:gid info) (user-supplementary-gids)))
+		    (zero? (bitwise-and acc (arithmetic-shift perms 3))))
 		      
-			(else				    ; Other
-			 (zero? (bitwise-and acc perms)))))
-		'permission)))))
+		   (else		; Other
+		    (zero? (bitwise-and acc perms)))))
+	   'permission))))
 
 ;;;;;;
 
