@@ -1,17 +1,18 @@
 ;;; Networking for the Scheme Shell
 ;;; Copyright (c) 1994-1995 by Brian D. Carlstrom.
 ;;; Copyright (c) 1994 by Olin Shivers.
+;;; See file COPYING.
 
 ;;; Scheme48 implementation.
 
 (foreign-source
  "#include <sys/types.h>"
  "#include <sys/socket.h>"
+ "#include <errno.h>"
  ""
  "/* Make sure foreign-function stubs interface to the C funs correctly: */"
  "#include \"network1.h\""
  ""
- "extern int errno;"
  "extern int h_errno;"
  ""
  "#define errno_on_zero_or_false(x) ((x) ? SCHFALSE : ENTER_FIXNUM(errno))"
@@ -47,35 +48,40 @@
 		     (else 
 		      (error "socket-connect: unsupported protocol-family ~s"
 			     protocol-family)))))
-    (connect-socket sock addr)
-    sock))
+    ;; Close the socket and free the file-descriptors
+    ;; if the connect fails:
+    (let ((connected #f))
+      (dynamic-wind
+       (lambda () #f)
+       (lambda () (connect-socket sock addr) (set! connected #t))
+       (lambda ()
+         (if (not connected)
+             (close-socket sock))))
+      (if connected
+          sock
+          #f))))
 
 (define (bind-listen-accept-loop protocol-family proc arg)
-  (let* ((sock (create-socket protocol-family socket-type/stream))
-	 (addr (cond ((= protocol-family
-			 protocol-family/internet)
-		      (let ((port (cond ((integer? arg) arg)
-					((string? arg)
-					 (service-info:port 
-					  (service-info arg "tcp")))
-					(else
-					 (error "socket-connect: bad arg ~s"
-						arg)))))
-			(internet-address->socket-address internet-address/any
-							  arg)))
-		     ((= protocol-family
-			 protocol-family/unix)
-		      (unix-address->socket-address arg))
-		     (else 
-      (error "bind-listen-accept-loop: unsupported protocol-family ~s"
-	     protocol-family)))))
+  (let ((sock (create-socket protocol-family socket-type/stream))
+	(addr (cond ((= protocol-family protocol-family/internet)
+		     (internet-address->socket-address internet-address/any
+		       (cond ((integer? arg) arg)
+			     ((string? arg)
+			      (service-info:port (service-info arg "tcp")))
+			     (else (error "socket-connect: bad arg ~s" arg)))))
+
+		    ((= protocol-family protocol-family/unix)
+		     (unix-address->socket-address arg))
+
+		    (else
+		     (error "bind-listen-accept-loop: unsupported protocol-family ~s"
+			    protocol-family)))))
+
     (set-socket-option sock level/socket socket/reuse-address #t)
     (bind-socket sock addr)
     (listen-socket sock 5)
     (let loop ()
-      (call-with-values 
-       (lambda () (accept-connection sock))
-       proc)
+      (call-with-values (lambda () (accept-connection sock)) proc)
       (loop))))
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
@@ -238,6 +244,7 @@
 (define-foreign %listen/errno
   (listen (integer sockfd)	; socket fdes
 	  (integer backlog))	; backlog
+	no-declare ; for Linux
   (to-scheme integer errno_or_false))
 
 (define-errno-syscall (%listen sockfd backlog) %listen/errno)
