@@ -9,7 +9,9 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/socket.h>
+#ifdef HAVE_SYS_UN_H
 #include <sys/un.h>
+#endif
 #include <errno.h>
 #include <netdb.h>
 #include <netinet/in.h>
@@ -20,7 +22,9 @@
 /* Make sure our exports match up w/the implementation: */
 #include "network1.h"
 
+#if !defined(__CYGWIN__) && !defined(_AIX)
 extern int h_errno;
+#endif
 
 /* to extract a 4 byte long value from a scheme string */
 
@@ -33,6 +37,7 @@ int scheme_bind(int sockfd, int family, scheme_value scheme_name)
 {
   switch(family)
     {
+#ifdef HAVE_SYS_UN_H
     case AF_UNIX: 
       {
 	struct sockaddr_un name;
@@ -48,6 +53,7 @@ int scheme_bind(int sockfd, int family, scheme_value scheme_name)
 	return(bind(sockfd,(struct sockaddr *)&name,sizeof(name)));
 	break;
       }
+#endif
     case AF_INET: 
       {
 	struct sockaddr_in name;
@@ -70,6 +76,7 @@ int scheme_connect(int sockfd, int family, scheme_value scheme_name)
 {
   switch(family)
     {
+#ifdef HAVE_SYS_UN_H
     case AF_UNIX: 
       {
 	struct sockaddr_un name;
@@ -85,6 +92,7 @@ int scheme_connect(int sockfd, int family, scheme_value scheme_name)
 	return(connect(sockfd,(struct sockaddr *)&name,sizeof(name)));
 	break;
       }
+#endif
     case AF_INET: 
       {
 	struct sockaddr_in name;
@@ -111,6 +119,7 @@ int scheme_accept(int sockfd, int family, scheme_value scheme_name)
 {
   switch(family)
     {
+#ifdef HAVE_SYS_UN_H
     case AF_UNIX: 
       {
 	struct sockaddr_un name;
@@ -123,7 +132,8 @@ int scheme_accept(int sockfd, int family, scheme_value scheme_name)
 	return(newsockfd);
 	break;
       }
-    case AF_INET: 
+#endif
+      case AF_INET: 
       {
 	struct sockaddr_in name;
 	int namelen=sizeof(name);
@@ -254,6 +264,7 @@ int send_substring(int s,
       {
 	return send(s, StrByte(buf,start), end-start, flags);
       }
+#ifdef HAVE_SYS_UN_H
     case AF_UNIX:
       {
 	struct sockaddr_un name;
@@ -272,7 +283,8 @@ int send_substring(int s,
 		      (struct sockaddr *)&name, sizeof(name)));
 	break;
       }
-    case AF_INET:
+#endif
+      case AF_INET:
       {
 	struct sockaddr_in name;
 	u_long  addr=GET_LONG(scheme_name,0);
