@@ -1,6 +1,7 @@
 ;;; The module interfaces for scsh.
 ;;; Copyright (c) 1994 by Olin Shivers and David Albertz.
 ;;; Copyright (c) 1994 by Brian D. Carlstrom
+;;; See file COPYING.
 
 (define-interface posix-fdflags-interface
   (export ((open/read
@@ -96,8 +97,9 @@
 (define-interface scsh-errors-interface
   (export errno-error
 	  error
-	  (with-errno-handler* (proc ((proc (:number :value) :values)); handler
-				     (proc () :values)))	      ; thunk
+	  (with-errno-handler* (proc ((proc (:number :value) :values); handler
+				      (proc () :values))	     ; thunk
+				     :values))
 	  (with-errno-handler :syntax)))
 
 
@@ -207,7 +209,7 @@
 			      port->sexp-list
 			      port->string-list
 			      port->list
-			      reduce-port
+			      port-fold reduce-port
 			      port->fdes
 			      read-string
 			      read-string!
@@ -282,7 +284,7 @@
 	  directory-files
 	  glob
 	  glob-quote
-	  file-match
+;	  file-match
 
 	  create-temp-file
 	  temp-file-iterate
@@ -417,6 +419,7 @@
 	  exec-path-list))
 
 
+;;; Kill me?
 (define-interface scsh-regexp-interface
   (export string-match
 	  regexp-match?
@@ -424,15 +427,18 @@
 	  match:end
 	  match:substring
 	  make-regexp
+	  ->regexp
 	  regexp?
-	  regexp-exec
+	  regexp-search
 	  regexp-substitute
 	  regexp-substitute/global
 	  regexp-quote))
 
 
 (define-interface scsh-string-interface
-  (export substitute-env-vars index rindex))
+  (export substitute-env-vars
+	  ;string-index string-index-right ; Now in string-lib
+	  ))
 
 (define-interface scsh-file-names-interface
   (export file-name-as-directory
@@ -452,6 +458,7 @@
 	  simplify-file-name
 	  resolve-tilde-file-name
 	  resolve-file-name
+	  absolute-file-name
 	  home-dir
 	  home-file))
 
@@ -484,6 +491,18 @@
 	  set-date:week-day	
 	  set-date:year-day
 
+	  modify-date:seconds
+	  modify-date:minute	
+	  modify-date:hour   	
+	  modify-date:month-day	
+	  modify-date:month   	
+	  modify-date:year    	
+	  modify-date:tz-name	
+	  modify-date:tz-secs	
+	  modify-date:summer?	
+	  modify-date:week-day	
+	  modify-date:year-day
+
 	  time+ticks
 	  ticks/sec
 	  time
@@ -552,13 +571,15 @@
 
 
 (define-interface scsh-utilities-interface
-  (export del delete index rindex reduce filter first any first? nth
-	  any? every? mapv mapv! vector-every? copy-vector initialize-vector
+  (export del delete filter first first? nth
+	  fold fold-right
+	  any every
+	  mapv mapv! vector-every? copy-vector initialize-vector vector-append
+	  vfold vfold-right
 	  check-arg conjoin disjoin negate compose reverse! call/cc
 	  deprecated-proc
 	  deposit-bit-field
-	  real->exact-integer
-	  string-replace! substring-replace!))
+	  real->exact-integer))
 
 ;;; semi-standard network magic numbers
 ;;; should be available on all platforms
@@ -663,46 +684,8 @@
 	  ))
 
 
-(define-interface char-set-interface
-  (export char:newline char:tab char:linefeed char:page char:return char:space
-	  char-ascii?
-	  char-set?
-
-	  char-set
-	  chars->char-set
-	  string->char-set
-	  ascii-range->char-set
-	  predicate->char-set
-	  ->char-set
-
-	  char-set-members
-	  char-set-contains?
-
-	  char-set-invert
-	  char-set-union
-	  char-set-intersection
-	  char-set-difference
-
-	  char-set:upper-case
-	  char-set:lower-case
-	  char-set:numeric
-	  char-set:whitespace
-	  char-set:not-whitespace
-	  char-set:alphabetic
-	  char-set:alphanumeric
-	  char-set:graphic
-
-	  char-upper-case?
-	  char-lower-case?
-	  char-numeric?
-	  char-whitespace?
-	  char-alphabetic?
-	  char-alphanumeric?
-	  char-graphic?))
-
-
 (define-interface scsh-field-reader-interface
-  (export join-strings
+  (export join-strings	; Obsolete. Has become SRFI-13's STRING-JOIN.
 	  field-splitter infix-splitter suffix-splitter sloppy-suffix-splitter
 	  record-reader
 	  field-reader
@@ -716,7 +699,8 @@
 	  skip-char-set))
 
 (define-interface awk-interface
-  (export (awk :syntax)))
+  (export (awk :syntax)
+	  (awk/posix-string :syntax)))
 
 (define-interface scsh-dbm-interface
   (export dbm-open
@@ -955,6 +939,16 @@
 	  tty-info:min			set-tty-info:min
 	  tty-info:time			set-tty-info:time
 
+	  modify-tty-info:control-chars
+	  modify-tty-info:input-flags 
+	  modify-tty-info:output-flags  
+	  modify-tty-info:control-flags
+	  modify-tty-info:local-flags
+	  modify-tty-info:input-speed
+	  modify-tty-info:output-speed
+	  modify-tty-info:min
+	  modify-tty-info:time
+
 	  make-tty-info			copy-tty-info
 
 	  tty-info
@@ -1021,3 +1015,25 @@
 	  interrupt/winch
 	  interrupt/xcpu	
 	  interrupt/xfsz))
+
+(define-interface odbc-interface
+  (export current-db
+	  call/db
+	  with-current-db*
+	  with-current-db
+	  with-open-db*
+	  with-open-db
+	  string->sql-command
+	  open-db
+	  set-current-db!
+	  close-db
+ 	  execute-sql
+	  fetch-row
+	  fetch-rows
+	  fetch-all
+	  close-cursor
+	  commit
+	  rollback
+	  cancel
+	  cursor-name?
+	  set-cursor-name!))
