@@ -1,5 +1,5 @@
 ;;; The packages that scsh uses/defines.
-;;; Copyright (c) 1994 by Olin Shivers.
+;;; Copyright (c) 1994 by Olin Shivers. See file COPYING.
 
 ;;; Note: field-reader package (fr.scm) and here docs use READ-LINE.
 ;;; It is defined in rdelim.scm.
@@ -27,12 +27,16 @@
 
 
 (define-structure error-package (export error warn)
-  (open signals))
+  (open signals)
+  (optimize auto-integrate)
+  )
 
 
 (define-structure scsh-utilities scsh-utilities-interface
-  (open bitwise error-package let-opt scheme)
-  (files utilities))
+  (open bitwise error-package loopholes let-opt scheme)
+  (files utilities)
+  (optimize auto-integrate)
+  )
 
 
 ;;; This guy goes into the FOR-SYNTAX part of scsh's syntax exports.
@@ -44,24 +48,25 @@
 	scsh-utilities	; check-arg
 	scheme
 	)
-  (files syntax-helpers))
+  (files syntax-helpers)
+  (optimize auto-integrate)
+  )
 
 
 ;;; The bufpol/{block, line, none} values
 (define-structure buffered-io-flags buffered-io-flags-interface
   (open defenum-package scheme)
-  (files (machine bufpol)))
-
-
-(define-structure char-set-package char-set-interface  
-  (open error-package ascii scheme)
-  (files char-set))
+  (files (machine bufpol))
+  (optimize auto-integrate)
+  )
 
 
 (define-structures ((tty-flags tty-flags-interface)
 		    (scsh-internal-tty-flags scsh-internal-tty-flags-interface))
   (open scheme ascii bitwise)
-  (files (machine tty-consts)))
+  (files (machine tty-consts))
+  (optimize auto-integrate)
+  )
 
 
 (define-structure scsh-version scsh-version-interface
@@ -76,7 +81,9 @@
 	  $current-input-port
 	  $current-output-port
 	  $error-output-port)
-  (open ports))
+  (open ports)
+  (optimize auto-integrate)
+  )
 
 ;;; The scsh-level-0 package is for implementation convenience.
 ;;; The scsh startup and top-level modules need access to scsh
@@ -90,8 +97,8 @@
     (compound-interface posix-fdflags-interface
 			posix-errno-interface
 			posix-signals-interface
-			sockets-network-interface ; Standard Network Interface
-			os-extras-interface ; Extra stuff from OS.
+			sockets-network-interface   ; Standard Network Interface
+			os-extras-interface         ; Extra stuff from OS.
 			scsh-delimited-readers-interface
 			scsh-errors-interface
 			scsh-io-interface
@@ -107,21 +114,22 @@
 			scsh-file-names-interface
 			scsh-misc-interface
 			scsh-high-level-process-interface
-			scsh-time-interface ; new in 0.2
-			scsh-sockets-interface ; new in 0.3
-			tty-interface ; new in 0.4
+			scsh-time-interface         ; New in release 0.2
+			scsh-sockets-interface      ; New in release 0.3
+			tty-interface               ; New in release 0.4
 			scsh-version-interface
 			char-set-interface
 			signal-handler-interface
 			;; This stuff would probably be better off kept
 			;; in separate modules, but we'll toss it in for now.
-			(interface-of ascii) ; char<->ascii
+			(interface-of ascii)        ; char<->ascii
 			string-ports-interface
 			))
    (scsh-level-0-internals (export set-command-line-args!
 				   init-scsh-hindbrain
 				   init-scsh-vars))
-   (scsh-regexp-package scsh-regexp-interface))
+; (scsh-regexp-package scsh-regexp-interface)
+   )
   (for-syntax (open scsh-syntax-helpers scheme))
   (open externals
 	structure-refs
@@ -130,8 +138,8 @@
 	defrec-package
 	define-foreign-syntax
 	formats
-	os-dependent		; OS dependent stuff
-	buffered-io-flags	; stdio dependent
+	os-dependent                        ; OS dependent cpde
+	buffered-io-flags                   ; stdio dependent
 	ascii
 	records
 	extended-ports
@@ -147,25 +155,34 @@
 	fluids
 	weak
 
+	scsh-char-set-low-level-lib	    ; rdelim.scm needs it.
 ;	scsh-regexp-package
 ;	scsh-regexp-internals
-	char-set-package
+	char-set-lib
 	scsh-version
 	tty-flags
-	scsh-internal-tty-flags	; Not exported
-	let-opt			; optional-arg parsing & defaulting
+	scsh-internal-tty-flags             ; Not exported
+	let-opt                             ; optional-arg parsing & defaulting
 	
-	interrupts	; signal handler code
+	interrupts                          ; signal handler code
+
+	re-level-0
+	rx-syntax
+
+	string-lib
+
+	fluids			; For exec-path-list
+	loopholes		; For my bogus CALL-TERMINALLY implementation.
 
 	scheme
 	)
 
   (access command-processor
 	  escapes
-	  ports		; S48's force-output
+	  ports                             ; S48's force-output
 	  formats
-	  records	; I don't think this is necessary. !!!
-	  scheme)	; For accessing the normal I/O operators.
+	  records                           ; I don't think this is necessary. !!!
+	  scheme)                         ; For accessing the normal I/O operators.
 
   (begin (define set-batch-mode?!
 	   (structure-ref command-processor set-batch-mode?!))
@@ -180,34 +197,40 @@
 	 rw
 	 newports
 	 fdports
-	 procobj		; New in release 0.4.
-	 (machine waitcodes)	; OS dependent code.
+	 procobj                            ; New in release 0.4
+	 (machine waitcodes)                ; OS dependent code
 	 filesys
 	 fileinfo
 	 glob
-	 filemtch
-	 time			; New in release 0.2.
+;	 filemtch
+	 time                               ; New in release 0.2
 	 (machine time_dep)
-	 network		; New in release 0.3.
-	 endian			; New in release 0.4.
-	 flock			; New in release 0.4.
-	 tty			; New in release 0.4.
-	 pty			; New in release 0.4.
-	 sighandlers		; New in release 0.5.
+	 network                            ; New in release 0.3
+	 endian                             ; New in release 0.4
+	 flock                              ; New in release 0.4
+	 tty                                ; New in release 0.4
+	 pty                                ; New in release 0.4
+	 sighandlers                        ; New in release 0.5
 	 scsh
-	 re
+;	 re
 	 rdelim
-	 ))
+	 )
+;  (optimize auto-integrate)
+  )
 
 (define-structure defrec-package (export (define-record :syntax))
   (open records scheme)
   (for-syntax (open scheme error-package receiving))
-  (files defrec))
+  (files defrec)
+  (optimize auto-integrate)
+  )
 
 (define-structure defenum-package (export (define-enum-constant  :syntax)
 					  (define-enum-constants :syntax))
   (open scheme)
-  (files enumconst))
+  (files enumconst)
+  (optimize auto-integrate)
+  )
 
 ;;; This code opens so many modules of gruesome, low-level S48 internals 
 ;;; that these two modules are segregated into separate packages, each
@@ -226,7 +249,9 @@
 	filenames		; translate
 	scheme-level-2-internal	; usual-resumer
 	scheme)
-  (files startup))
+  (files startup)
+  (optimize auto-integrate)
+  )
 
 (define-structure scsh-top-package (export parse-switches-and-execute repl)
   (open command-processor
@@ -242,66 +267,119 @@
 	packages
 	receiving
 	scsh-version
-	scsh-level-0		; with-current-input-port error-output-port
-				; with-current-output-port exit
-	scsh-level-0-internals	; set-command-line-args! init-scsh-vars
+	scsh-level-0                    ; with-current-input-port error-output-port
+                                    ; with-current-output-port exit
+	scsh-level-0-internals          ; set-command-line-args! init-scsh-vars
+	handle list-lib                 ; For lib-search facility
 	scheme)
-  (files top meta-arg))
+  (files top meta-arg)
+  (optimize auto-integrate)
+  )
 
 
 (define-structure field-reader-package scsh-field-reader-interface
   (open receiving		; receive
-	char-set-package
-	scsh-utilities
+	scsh-utilities		; nth & deprecated-proc (for join-strings)
 	error-package		; error
+	string-lib		; string-join for obsolete join-strings
 	scsh-level-0		; delimited readers
-	scsh-regexp-package
+;	scsh-regexp-package
+	re-exports
 	let-opt			; optional-arg parsing & defaulting
 	scheme
 	)
-  (files fr))
+  (files fr)
+  ;; Handle a little bit of backwards compatibility.
+  (begin (define join-strings (deprecated-proc string-join 'join-strings
+					       "Use SRFI-13 STRING-JOIN.")))
+  (optimize auto-integrate)
+  )
 
 
 (define-structures
-  ((awk-expander-package (export expand-awk))
+  ((awk-expander-package (export expand-awk expand-awk/obsolete))
    (awk-support-package (export next-range next-:range
 				next-range: next-:range:)))
   (open receiving		; receive
 	scsh-utilities	
 	error-package		; error
-	scsh-regexp-package
+;	scsh-regexp-package
+;	re-exports
+	sre-syntax-tools
 	scheme
 	)
-  (files awk))
+  (files awk)
+  (optimize auto-integrate)
+  )
 	 
 
 (define-structure awk-package awk-interface
-  (open awk-support-package scsh-regexp-package receiving scheme)
+  (open awk-support-package	; These packages provide all the stuff 
+	re-exports		; that appears in the code produced by 
+	receiving		; an awk expansion.
+	scheme)
   (for-syntax (open awk-expander-package scheme))
-  (begin (define-syntax awk expand-awk)))
+  (begin (define-syntax awk expand-awk)
+	 (define-syntax awk/posix-string expand-awk/obsolete)))
 
+;;; Exports an AWK macro that is just AWK/POSIX-STRING.
+(define-structure obsolete-awk-package (export (awk :syntax))
+  (open awk-package)
+  (begin (define-syntax awk 
+           (syntax-rules () ((awk body ...) (awk/posix-string body ....))))))
+
+(define-structure odbc-package odbc-interface
+  (open scsh-level-0
+        defrec-package
+        define-foreign-syntax
+        let-opt
+        conditions
+        fluids
+        handle
+        signals
+        scheme)
+  (files
+	 (odbc odbc0)                       ; New in release 0.5.3
+	 (odbc odbc1)                       ; New in release 0.5.3
+	 (odbc odbc2)                       ; New in release 0.5.3
+     )
+;  (optimize auto-integrate)
+)
 
 (define-structure scsh
   (compound-interface (interface-of scsh-level-0)
 		      (interface-of scsh-startup-package)
-		      scsh-regexp-interface
+;		      scsh-regexp-interface
+		      re-exports-interface
+		      re-old-funs-interface
 		      scsh-field-reader-interface 	; new in 0.3
 ;		      scsh-dbm-interface
 		      (export repl)
-		      awk-interface)
+		      awk-interface
+		      odbc-interface
+		      char-predicates-interface; Urk -- Some of this is R5RS!
+		      obsolete-char-set-interface
+		      )
 
   (open structure-refs
+	obsolete-char-set-lib
 	scsh-level-0
 	scsh-level-0-internals
-	scsh-regexp-package
+	re-exports
+	re-old-funs
+;	scsh-regexp-package
 	scsh-startup-package
 ;	dbm
 	awk-package
+	odbc-package
 	field-reader-package
+	char-predicates-lib	; Urk -- Some of this is R5RS!
 	scheme)
 
   (access scsh-top-package)
-  (begin (define repl (structure-ref scsh-top-package repl))))
+  (begin (define repl (structure-ref scsh-top-package repl)))
+  (optimize auto-integrate)
+  )
 
 (define-structure scsh-here-string-hax (export)
   (open reading
@@ -309,11 +387,6 @@
 	scsh		; Just need the delimited readers.
 	features	; make-immutable!
 	scheme)
-  (files here))
-
-(define-structure test-package (export test-proc)
-  (open scsh-regexp-package scheme)
-  (begin (define (test-proc p)
-	   (regexp-substitute p
-			      (string-match "(foo)(.*)(bar)" "Hello foo Olin bar quux")
-			      'post 3 1 2 'pre))))
+  (files here)
+  (optimize auto-integrate)
+  )
