@@ -1,13 +1,15 @@
 ;;; A Scheme shell.
-;;; Copyright (c) 1992 by Olin Shivers.
+;;; Copyright (c) 1992-1999 by Olin Shivers.
 ;;; Copyright (c) 1994 by Brian D. Carlstrom.
+;;; See file COPYING.
 
 ;;; Call THUNK, then die.
 ;;; A clever definition in a clever implementation allows the caller's stack
 ;;; and dynamic env to be gc'd away, since this procedure never returns.
 
 (define (call-terminally thunk)
-  (with-continuation #f (lambda () (thunk) (exit 0))))
+  (with-continuation (loophole :escape #f)	; Bogus
+		     (lambda () (thunk) (exit 0))))
   ;; Alternatively: (with-continuation #f thunk)
 
 ;;; More portably, but less usefully:
@@ -125,7 +127,7 @@
 
 	;; Main loop.
 	(let split ((i 0))
-	  (cond ((index clist #\: i) =>
+	  (cond ((string-index clist #\: i) =>
 		 (lambda (colon)
 		   (cons (substring clist i colon)
 			 (split (+ colon 1)))))
@@ -207,10 +209,10 @@
 
 (define (with-env* alist-delta thunk)
   (let* ((old-env #f)
-	 (new-env (reduce (lambda (alist key/val)
-			    (alist-update (car key/val) (cdr key/val) alist))
-			  (env->alist)
-			  alist-delta)))
+	 (new-env (fold (lambda (key/val alist)
+			  (alist-update (car key/val) (cdr key/val) alist))
+			(env->alist)
+			alist-delta)))
     (dynamic-wind
       (lambda ()
 	(set! old-env (env->alist))
@@ -295,7 +297,8 @@
 	     (close-fdes (open-fdes fname oflags #o600))
 	     fname)
 	   (if (null? maybe-prefix) '()
-	       (list (string-append (car maybe-prefix) ".~a"))))))
+	       (list (string-append (constant-format-string (car maybe-prefix))
+				    ".~a"))))))
 
 (define *temp-file-template*
   (make-fluid (string-append "/usr/tmp/" (number->string (pid)) ".~a")))
@@ -314,6 +317,23 @@
 		  (loop (+ i 1)))))))))
 
 
+;; Double tildes in S. 
+;; Using the return value as a format string will output exactly S.
+(define (constant-format-string s)	; Ugly code. Would be much clearer
+  (let* ((len (string-length s))	; if written with string SRFI.
+	 (tilde? (lambda (s i) (char=? #\~ (string-ref s i))))
+	 (newlen (do ((i (- len 1) (- i 1))
+		      (ans 0 (+ ans (if (tilde? s i) 2 1))))
+		     ((< i 0) ans)))
+	 (fs (make-string newlen)))
+    (let lp ((i 0) (j 0))
+      (cond ((< i len)
+	     (let ((j (cond ((tilde? s i) (string-set! fs j #\~) (+ j 1))
+			    (else j))))
+	       (string-set! fs j (string-ref s i))
+	       (lp (+ i 1) (+ j 1))))))
+    fs))
+
 
 ;;; Roughly equivalent to (pipe).
 ;;; Returns two file ports [iport oport] open on a temp file.
@@ -435,12 +455,12 @@
 ;;; (port->list reader port)
 ;;;     Repeatedly applies READER to PORT, accumulating results into a list.
 ;;;     On EOF, returns the list of items thus collected.
-;;; (reduce-port port reader op . seeds)
+;;; (port-fold port reader op . seeds)
 ;;;     Repeatedly read things from PORT with READER. Each time you read
 ;;;     some value V, compute a new set of seeds with (apply OP V SEEDS).
 ;;;     (More than 1 seed means OP must return multiple values).
-;;;     On eof, return the seeds.
-;;;     PORT->LIST is just (REDUCE-PORT PORT READ CONS '())
+;;;     On eof, return the seeds: (apply value SEEDS).
+;;;     PORT->LIST is just (PORT-FOLD PORT READ CONS '())
 
 (define (run/port+proc* thunk)
   (receive (r w) (pipe)
@@ -505,13 +525,16 @@
 (define (port->string-list port)
   (port->list read-line port))
 
-(define (reduce-port port reader op . seeds)
-  (letrec ((reduce (lambda seeds
+(define (port-fold port reader op . seeds)
+  (letrec ((fold (lambda seeds
 		     (let ((x (reader port)))
 		       (if (eof-object? x) (apply values seeds)
 			   (call-with-values (lambda () (apply op x seeds))
-					     reduce))))))
-    (apply reduce seeds)))
+					     fold))))))
+    (apply fold seeds)))
+
+(define reduce-port
+  (deprecated-proc port-fold 'reduce-port "Use port-fold instead."))
 
 ;;; Not defined:
 ;;; (field-reader field-delims record-delims)
@@ -622,7 +645,7 @@
       (arg* arglist n)))
 
 (define (argv n . maybe-default)
-  (apply arg (cdr %command-line) n maybe-default))
+  (apply arg %command-line (+ n 1) maybe-default))
 
 (define (command-line) (append %command-line '()))
 
@@ -659,7 +682,7 @@
   (%exec prog (cons prog arglist) env))
 
 ;(define (exec-path/env prog env . arglist)
-;  (cond ((exec-path-search (stringify prog) exec-path-list) =>
+;  (cond ((exec-path-search (stringify prog) (fluid exec-path-list)) =>
 ;	 (lambda (binary)
 ;	   (apply exec/env binary env arglist)))
 ;	(else (error "No executable found." prog arglist))))
@@ -671,7 +694,7 @@
 (define (exec-path/env prog env . arglist)
   (flush-all-ports)
   (let ((prog (stringify prog)))
-    (if (index prog #\/)
+    (if (string-index prog #\/)
 
 	;; Contains a slash -- no path search.
 	(%exec prog (cons prog arglist) env)
@@ -681,7 +704,7 @@
 	  (for-each (lambda (dir)
 		      (let ((binary (string-append dir "/" prog)))
 			(%%exec/errno binary argv env)))
-		    exec-path-list))))
+		    (fluid exec-path-list)))))
 
     (error "No executable found." prog arglist))
 	 
@@ -741,7 +764,7 @@
 
 ;;; Some globals:
 (define home-directory "")
-(define exec-path-list '())
+(define exec-path-list (make-fluid '()))
 
 (define (init-scsh-vars quietly?)
   (set! home-directory
@@ -749,11 +772,11 @@
 	      (else (if (not quietly?)
 			(warn "Starting up with no home directory ($HOME)."))
 		    "/")))
-  (set! exec-path-list
-	(cond ((getenv "PATH") => split-colon-list)
-	      (else (if (not quietly?)
-			(warn "Starting up with no path ($PATH)."))
-		    '()))))
+  (set-fluid! exec-path-list
+	      (cond ((getenv "PATH") => split-colon-list)
+		    (else (if (not quietly?)
+			      (warn "Starting up with no path ($PATH)."))
+			  '()))))
 
 
 ; SIGTSTP blows s48 away. ???
