@@ -1,5 +1,5 @@
 ;;; POSIX system-call Scheme binding.
-;;; Copyright (c) 1993 by Olin Shivers.
+;;; Copyright (c) 1993 by Olin Shivers. See file COPYING.
 
 ;;; Scheme48 implementation.
 
@@ -12,6 +12,7 @@
   "#include <sys/time.h>"
   "#include <fcntl.h>		/* for O_RDWR */" ; ???
   "#include <sys/stat.h>"
+  "#include <errno.h>"
   "#include <netdb.h>"
   "#include <pwd.h>"
   "#include <unistd.h>"
@@ -23,8 +24,6 @@
   "#include \"syscalls1.h\""
   "#include \"userinfo1.h\""
   ""
-  "extern int errno;"
-  ""
   "#define errno_on_zero_or_false(x) ((x) ? SCHFALSE : ENTER_FIXNUM(errno))"
   "#define errno_or_false(x) (((x) == -1) ? ENTER_FIXNUM(errno) : SCHFALSE)"
   "#define False_on_zero(x) ((x) ? ENTER_FIXNUM(x) : SCHFALSE)" ; Not a function.
@@ -301,7 +300,7 @@
   (to-scheme integer errno_or_false))
 
 (define-foreign set-fdes-uid&gid/errno
-  (fchown (integer fd) (uid_t uid) (gid_t gid))
+  (fchown (integer fd) (uid_t uid) (gid_t gid)) no-declare ; for NT Cygwin
   (to-scheme integer errno_or_false))
 
 (define-errno-syscall (set-file-owner thing uid)
@@ -851,23 +850,26 @@
 ;;; I do this one in C, I'm not sure why:
 ;;; It is used by MATCH-FILES.
 
-(define-foreign %filter-C-strings!
-  (filter_stringvec (string pattern) ((C "char const ** ~a") cvec))
-  static-string	; error message -- #f if no error.
-  integer)	; number of files that pass the filter.
+;;; 99/7: No one is using this function, so I'm commenting it out.
+;;; Later, we could tune up the globber or regexp file-matcher to use
+;;; it (but should shift it into the rx directory). But I should also go
+;;; to a file-at-a-time cursor model for directory fetching. -Olin
 
+;(define-foreign %filter-C-strings!
+;  (filter_stringvec (string-desc pattern) ((C "char const ** ~a") cvec))
+;  integer)	; number of files that pass the filter.
 
-(define (match-files regexp . maybe-dir)
-  (let ((dir (:optional maybe-dir ".")))
-    (check-arg string? dir match-files)
-    (receive (err cvec numfiles)
-	     (%open-dir (ensure-file-name-is-nondirectory dir))
-      (if err (errno-error err match-files regexp dir))
-      (receive (err numfiles) (%filter-C-strings! regexp cvec)
-	(if err (error err match-files))
-	(%sort-file-vector cvec numfiles)
-	(let ((files (C-string-vec->Scheme&free cvec numfiles)))
-	  (vector->list files))))))
+;(define (match-files regexp . maybe-dir)
+;  (let ((dir (:optional maybe-dir ".")))
+;    (check-arg string? dir match-files)
+;    (receive (err cvec numfiles)
+;	     (%open-dir (ensure-file-name-is-nondirectory dir))
+;      (if err (errno-error err match-files regexp dir))
+;      (receive (numfiles) (%filter-C-strings! regexp cvec)
+;	;(if err (error err match-files))
+;	(%sort-file-vector cvec numfiles)
+;	(let ((files (C-string-vec->Scheme&free cvec numfiles)))
+;	  (vector->list files))))))
 
 
 ;;; Environment manipulation
@@ -876,7 +878,7 @@
 ;;; (var . val) / "var=val" rep conversion:
 
 (define (split-env-string var=val)
-  (let ((i (index var=val #\=)))
+  (let ((i (string-index var=val #\=)))
     (if i (values (substring var=val 0 i)
 		  (substring var=val (+ i 1) (string-length var=val)))
 	(error "No \"=\" in environment string" var=val))))
@@ -889,7 +891,10 @@
 
 (define (alist->env-vec alist)
   (list->vector (map (lambda (var.val)
-		       (string-append (car var.val) "=" (cdr var.val)))
+		       (string-append (car var.val) "="
+				      (let ((val (cdr var.val)))
+					(if (string? val) val
+					    (string-join val ":")))))
 		     alist)))
 
 ;;; ENV->ALIST
