@@ -55,8 +55,10 @@ extern char **environ;
 static char *utc_env[] = {"TZ=UCT0", 0};
 
 #ifdef HAVE_TZNAME
+#ifndef __CYGWIN__
 extern char *tzname[];	/* Why isn't this defined in time.h? */
 #endif
+#endif
 
 /* These two functions allow you to temporarily override
 ** the current time zone with one of your choice. make_newenv()
@@ -174,10 +176,12 @@ scheme_value time2date(int hi_secs, int lo_secs, scheme_value zone,
 #ifdef HAVE_GMTOFF	
 	*tz_secs = d.tm_gmtoff;
 #else
-	{ char **oldenv = environ;			/* Set TZ to UTC     */
+	{ struct tm dcopy = d;
+	  char **oldenv = environ;			/* Set TZ to UTC     */
 	  environ=utc_env;				/* time temporarily. */
 	  tzset(); /* NetBSD, SunOS POSIX-noncompliance requires this. */
-	  *tz_secs = mktime(&d) - t;
+	  dcopy.tm_isdst = 0;
+	  *tz_secs = mktime(&dcopy) - t;	/* mktime() may mung dcopy. */
 	  environ=oldenv;
 	  }
 #endif
