@@ -3,7 +3,7 @@
 
 /*
    If you have concrete suggestions for improvements, they are quite
-   welcome.  Please send them to scheme-48-bugs@martigny.ai.mit.edu.
+   welcome.  Please send them to scheme-48-bugs@zurich.ai.mit.edu.
 
    Non-Posixisms:
 
@@ -195,12 +195,11 @@ char_ready_p( FILE* stream )
 #elif defined(__linux__)
   if (stream->_IO_read_ptr < stream->_IO_read_end)
     return 1;
-#elif defined(BSD) && (BSD >= 199306)
+#elif (defined(BSD) && (BSD >= 199306)) || defined(__CYGWIN32__)
   if (stream->_r)
     return 1;
 #else
-
-  /* Add new cases here AND SEND THEM TO scheme-48@martigny.ai.mit.edu
+  /* Add new cases here AND SEND THEM TO scheme-48@zurich.ai.mit.edu
      SO THAT THEY CAN GO INTO THE NEXT RELEASE!  (That means you, Olin.)
      It's generally pretty easy to figure out what to put here by
      examining /usr/include/stdio.h.  If the input stream's buffer is
@@ -217,7 +216,9 @@ char_ready_p( FILE* stream )
   {  fd_set readfds;
      FD_ZERO(&readfds);
      FD_SET(fileno(stream), &readfds);
+#ifndef __CYGWIN32__
      timerclear(&timeout);
+#endif
      return select1(FD_SETSIZE, &readfds, NULL, NULL, &timeout);
      }
 #else /* No select() - but there will generally be some other way to do this.*/
@@ -522,7 +523,7 @@ fill_the_table(void)
 	entry.n_scnum != N_UNDEF &&
 	entry.n_scnum != N_DEBUG)
       {
-	extern char *ldgetname(LDFILE *, SYMENT *);
+	extern char *ldgetname(LDFILE *, void *);
 	char *p = ldgetname(f, &entry);
 
 	if (p != NULL && p[0] == '.') {
@@ -638,7 +639,7 @@ lookup_external_name(char *name, long *location)
   strcpy(nm+1,name);
 #else
   strcpy(nm,name);
-#endif /*! HAVE_DLOPEN */
+#endif /*! USCORE */
 #if defined(HAVE_DLOPEN)
   return lookup_dlsym(nm, location);
 #elif defined(HAVE_NLIST)
